#include "controller.h"
#include "xparameters.h"
#include "ascii_characters.h"
#include "xio.h"
#include "stdio.h"
#include "xuartlite_l.h"
#include "xuartlite.h"

#include "w3_userio.h"
#include "w3_ad_controller.h"
#include "w3_clock_controller.h"
#include "w3_iic_eeprom.h"
#include "radio_controller.h"

//Abstracts away the long xparameters name
//This way, if we change the name of the core or register,
//we only have to change this line.
#define RC_BASEADDR 		XPAR_RADIO_CONTROLLER_0_BASEADDR
#define AD_BASEADDR 		XPAR_W3_AD_CONTROLLER_0_BASEADDR
#define CLK_BASEADDR 		XPAR_W3_CLOCK_CONTROLLER_0_BASEADDR
#define PHASEINC XPAR_DDS_EXAMPLE_PLBW_0_MEMMAP_PHASEINC


//Instantiates an instance of the UART driver (controls the serial)
static XUartLite UartLite;

int main(){
	w3_node_init();

	//Prints to the user's terminal
	xil_printf("******WARP DDS EXAMPLE*******\r\n");
	xil_printf("Press 1 to increase the phase increment\r\n");
	xil_printf("Press q to decrease the phase increment\r\n");

	//Set up the UART
	XUartLite_Initialize(&UartLite, XPAR_UARTLITE_0_DEVICE_ID);

	//Initial Configuration of DDS
	XIo_Out32(PHASEINC,(1<<15));	
	
	unsigned char x = 0;
	unsigned int phaseIncTemp;
	
	while(1){

		//Receives a character from the UART and sticks it in RecvBuffer
		x = XUartLite_RecvByte(STDIN_BASEADDRESS);

		if(x==ASCII_1){
				//Increase Phase Increment
				phaseIncTemp = XIo_In32(PHASEINC)<<1;
				if(phaseIncTemp!=0) XIo_Out32(PHASEINC,phaseIncTemp);
				if(phaseIncTemp!=0){
					XIo_Out32(PHASEINC,phaseIncTemp);
					xil_printf("Phase Increment: Hex: 0x%x\r\n", phaseIncTemp);
					xil_printf("                 Dec: %d\r\n", phaseIncTemp);
				}
		}
		
		if(x==ASCII_q){
				//Decrease Phase Increment
				phaseIncTemp = XIo_In32(PHASEINC)>>1;
				if(phaseIncTemp!=0){
					XIo_Out32(PHASEINC,phaseIncTemp);
					xil_printf("Phase Increment: Hex: 0x%x\r\n", phaseIncTemp);
					xil_printf("                 Dec: %d\r\n", phaseIncTemp);
				}
		}
	}
	return 0;
}



int w3_node_init() {

	int status;
	int ret = XST_SUCCESS;

	microblaze_enable_exceptions();

	//Initialize the AD9512 clock buffers (RF reference and sampling clocks)
	status = clk_init(CLK_BASEADDR, 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in clk_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the AD9963 ADCs/DACs for on-board RF interfaces
	ad_init(AD_BASEADDR, (RFA_AD_CS | RFB_AD_CS), 2);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in ad_init (%d)\n", status);
		ret = XST_FAILURE;
	}

	//Initialize the radio_controller core and MAX2829 transceivers for on-board RF interfaces
	status = radio_controller_init(RC_BASEADDR, (RC_RFA | RC_RFB), 1, 1);
	if(status != XST_SUCCESS) {
		xil_printf("w3_node_init: Error in radioController_initialize (%d)\n", status);
		ret = XST_FAILURE;
	}

	return ret;
}
